import lwsdk

reload (lwsdk)

__author__      = 'Ernest Chan'
__date__        = 'October 2015'
__copyright__   = 'Copyright (C) 2015'
__version__     = '1.0'
__maintainer__  = 'Ernest Chan'
__email__       = 'ernpchan@gmail.com'
__status__      = '1.0'
__lwver__       = '2015.3'

# console = lwsdk.LWPCoreConsole()
# console.clear()

lwII = lwsdk.LWInterfaceInfo()
lwUI = lwsdk.LWPanels()
lwSI = lwsdk.LWSceneInfo()

class ItemChain(lwsdk.IGeneric):
    def __init__(self,context):
        super(ItemChain,self).__init__()       

    def process(self,ga):
        # bitwise operation to get Parent In Place state
        # non-zero return means the option is On
        self.pipState = lwII.generalFlags & lwsdk.LWGENF_PARENTINPLACE

        # create UI panel
        panel = lwUI.create('Item Chain')
        
        self.choice = panel.hchoice_ctl('Parent In Place',['Yes','No'])
        self.choice.set_w(160)
        
        # set ui item choice state based on Parent In Place  
        # state so script mimics how user is working
        if self.pipState is 0:
            self.choice.set_int(1)
        else:
            self.choice.set_int(0)
        
        # check if user cancelled -> close panel
        if panel.open(lwsdk.PANF_BLOCKING | lwsdk.PANF_CANCEL) == 0:
            lwUI.destroy(panel)
        
        # user hit Ok -> execute code
        else:
            self.execute()
            
        return lwsdk.AFUNC_OK
    
    def execute(self):
        # variable to flip if Parent In Place has to be toggled
        currState = False
        
        # condition checks to see if user choice matches current Parent In Place state
        if self.pipState is 0 and self.choice.get_int() is 0 or self.pipState is not 0 and self.choice.get_int() is 1:
            lwsdk.command('ParentInPlace')
            # flip variable because we had to toggle Parent In Place
            currState = True
        
        # user is parenting in place
        if self.choice.get_int() is 0:
            # get current time
            userTime = lwII.curTime    
            # user is not at 0 so we should go to 0 before parenting items
            # to prevent unwanted pip keyframes from being created
            if lwII.curTime is not 0:
                lwsdk.command('GoToFrame 0')
        
        # get selected item(s)
        selected = lwII.selected_items()
        
        # array to hold itemIDs that are converted to strings
        itemIDArray = []
        
        # iterate through selected to populate itemIDArray
        for i in selected:
            itemIDArray.append(lwsdk.itemid_to_str(i).split(' ')[0])
        
        for count,i in enumerate(selected):
            # item is last item -> break loop because it's the first 
            # selected item and therefore nothing to parent it to
            if count == len(selected) - 1:
                break
            else:
                # select item to be parented
                lwsdk.command('SelectItem ' + lwsdk.itemid_to_str(i).split(' ')[0])
                # parent to next item in list
                lwsdk.command('ParentItem ' + itemIDArray[count + 1])
        
        # Parent In Place was toggled -> toggle back to restore user's original setting before script was run
        if currState is True:
            lwsdk.command('ParentInPlace')
        
        # reselect original user's item selection
        for i in itemIDArray:
            lwsdk.command('AddToSelection ' + i)
        
        # user was parenting in place
        if self.choice.get_int() is 0:
            # check to see if user was working at frame 0, if not restore where the user was before the script was run
            if lwII.curTime is not userTime:
                lwsdk.command('GoToFrame ' + str(userTime * lwSI.framesPerSecond))

ServerTagInfo = [
					( 'ItemChain', lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
					( 'ItemChain', lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH ),
					( 'Utilities/Python', lwsdk.SRVTAG_MENU | lwsdk.LANGID_USENGLISH )
				]

ServerRecord = { lwsdk.GenericFactory('ItemChain', ItemChain) : ServerTagInfo }